@@ -14,16 +14,17 @@ import shapely
 import shapely.affinity
 import shapely.ops
 from affine import Affine
+from ruamel.yaml import YAML, ruamel, Representer
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from shapely.geometry import shape
+from shapely.geometry.base import BaseGeometry
+
 from eodatasets3.model import (
     FileFormat,
     DatasetDoc,
     ODC_DATASET_SCHEMA_URL,
     StacPropertyView,
 )
-from ruamel.yaml import YAML, ruamel, Representer
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
-from shapely.geometry import shape
-from shapely.geometry.base import BaseGeometry
 
 
 def _format_representer(dumper, data: FileFormat):
@@ -105,14 +106,20 @@ def dump_yaml(output_yaml: Path, *docs: Mapping) -> None:
         yaml.dump_all(docs, stream)
 
 
+def dumps_yaml(stream, *docs: Mapping) -> None:
+    """Dump yaml through a stream, using the default serialisation settings."""
+    return _init_yaml().dump_all(docs, stream=stream)
+
+
 def load_yaml(p: Path) -> Dict:
     yaml = _init_yaml()
     with p.open() as f:
         return yaml.load(f)
 
 
-def loads_yaml(s: Union[Text, IO]) -> Iterable[Dict]:
-    return _init_yaml().load_all(s)
+def loads_yaml(stream: Union[Text, IO]) -> Iterable[Dict]:
+    """Dump yaml through a stream, using the default deserialisation settings."""
+    return _init_yaml().load_all(stream)
 
 
 def from_path(path: Path) -> DatasetDoc:
@@ -195,11 +202,41 @@ def _structure_as_shape(d, t):
 
 
 def to_doc(d: DatasetDoc) -> Dict:
-    return _to_doc(d, with_formatting=False)
+    """
+    Serialise a DatasetDoc to a dict
+
+    If you plan to write this out as a yaml file on disk, you're
+    better off with `to_formatted_doc()`.
+    """
+    doc = attr.asdict(
+        d,
+        recurse=True,
+        dict_factory=dict,
+        # Exclude fields that are the default.
+        filter=lambda attr, value: "doc_exclude" not in attr.metadata
+        and value != attr.default
+        # Exclude any fields set to None. The distinction should never matter in our docs.
+        and value is not None,
+        retain_collection_types=False,
+    )
+    doc["$schema"] = ODC_DATASET_SCHEMA_URL
+    if d.geometry is not None:
+        doc["geometry"] = shapely.geometry.mapping(d.geometry)
+    doc["id"] = str(d.id)
+    doc["properties"] = dict(d.properties)
+    return doc
 
 
 def to_formatted_doc(d: DatasetDoc) -> CommentedMap:
-    return _to_doc(d, with_formatting=True)
+    """Serialise a DatasetDoc to a yaml-serialisation-ready dict"""
+    doc = prepare_formatting(to_doc(d))
+    # Add user-readable names for measurements as a comment if present.
+    if d.measurements:
+        for band_name, band_doc in d.measurements.items():
+            if band_doc.alias and band_name.lower() != band_doc.alias.lower():
+                doc["measurements"].yaml_add_eol_comment(band_doc.alias, band_name)
+
+    return doc
 
 
 def _stac_key_order(key: str):
@@ -211,65 +248,81 @@ def _stac_key_order(key: str):
         return key
 
 
-def _to_doc(d: DatasetDoc, with_formatting: bool):
-    if with_formatting:
-        doc = CommentedMap()
-        doc.yaml_set_comment_before_after_key("$schema", before="Dataset")
-    else:
-        doc = {}
+def _eo3_key_order(keyval: str):
+    """
+    Order keys in an an EO3 document.
 
-    doc["$schema"] = ODC_DATASET_SCHEMA_URL
-    doc.update(
-        attr.asdict(
-            d,
-            recurse=True,
-            dict_factory=CommentedMap if with_formatting else dict,
-            # Exclude fields that are the default.
-            filter=lambda attr, value: "doc_exclude" not in attr.metadata
-            and value != attr.default
-            # Exclude any fields set to None. The distinction should never matter in our docs.
-            and value is not None,
-            retain_collection_types=False,
-        )
-    )
+    Suitable for sorted() func usage.
+    """
+    key, val = keyval
+    try:
+        i = _EO3_PROPERTY_ORDER.index(key)
+        if i == -1:
+            return 999
+        return i
+    except ValueError:
+        return 999
 
+
+# A logical, readable order for properties to be in a dataset document.
+_EO3_PROPERTY_ORDER = [
+    "$schema",
+    # Products / Types
+    "name",
+    "license",
+    "metadata_type",
+    "description",
+    "metadata",
+    # EO3
+    "id",
+    "label",
+    "product",
+    "locations",
+    "crs",
+    "geometry",
+    "grids",
+    "properties",
+    "measurements",
+    "accessories",
+    "lineage",
+]
+
+
+def prepare_formatting(d: Mapping) -> CommentedMap:
+    """
+    Format an eo3 dataset dict for human-readable yaml serialisation.
+
+    This will order fields, add whitespace, comments, etc.
+
+    Output is intended for ruamel.yaml.
+    """
     # Sort properties for readability.
-    # PyCharm '19 misunderstands the type of a `sorted(dict.items())`
-    # noinspection PyTypeChecker
+    doc = CommentedMap(sorted(d.items(), key=_eo3_key_order))
     doc["properties"] = CommentedMap(
         sorted(doc["properties"].items(), key=_stac_key_order)
     )
 
-    if d.geometry is not None:
-        doc["geometry"] = shapely.geometry.mapping(d.geometry)
-    doc["id"] = str(d.id)
+    # Whitespace
+    doc.yaml_set_comment_before_after_key("$schema", before="Dataset")
+    if "geometry" in doc:
+        # Set some numeric fields to be compact yaml format.
+        _use_compact_format(doc["geometry"], "coordinates")
+    if "grids" in doc:
+        for grid in doc["grids"].values():
+            _use_compact_format(grid, "shape", "transform")
 
-    if with_formatting:
-        if "geometry" in doc:
-            # Set some numeric fields to be compact yaml format.
-            _use_compact_format(doc["geometry"], "coordinates")
-        if "grids" in doc:
-            for grid in doc["grids"].values():
-                _use_compact_format(grid, "shape", "transform")
+    _add_space_before(
+        doc,
+        "label" if "label" in doc else "id",
+        "crs",
+        "properties",
+        "measurements",
+        "accessories",
+        "lineage",
+    )
 
-        # Add user-readable names for measurements as a comment if present.
-        if d.measurements:
-            for band_name, band_doc in d.measurements.items():
-                if band_doc.alias and band_name.lower() != band_doc.alias.lower():
-                    doc["measurements"].yaml_add_eol_comment(band_doc.alias, band_name)
-
-        _add_space_before(
-            doc,
-            "label" if "label" in doc else "id",
-            "crs",
-            "properties",
-            "measurements",
-            "accessories",
-            "lineage",
-        )
-
-        p: CommentedMap = doc["properties"]
-        p.yaml_add_eol_comment("# Ground sample distance (m)", "eo:gsd")
+    p: CommentedMap = doc["properties"]
+    p.yaml_add_eol_comment("# Ground sample distance (m)", "eo:gsd")
 
     return doc
 
